# -*- coding: utf-8 -*-

"""Common widgets and their bases."""

__all__ = ['IconButton']

from qtpy import QtWidgets


class IconButton(QtWidgets.QPushButton):
    """Common control for icon-only buttons."""



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Conda Packager Manager Widget."""

# yapf: disable

from __future__ import (absolute_import, division, print_function,
                        with_statement)

# Standard library imports
import gettext
import sys

# Third party imports
from qtpy.QtCore import QEvent, QSize, Qt, Signal
from qtpy.QtWidgets import (QApplication, QDialogButtonBox, QHBoxLayout,
                            QProgressBar, QPushButton, QVBoxLayout, QWidget)

# Local imports
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.config import CONF
from anaconda_navigator.utils import constants as C
from anaconda_navigator.utils.analytics import GATracker
from anaconda_navigator.utils.logs import logger
from anaconda_navigator.utils.styles import load_style_sheet
from anaconda_navigator.widgets import (ButtonBase, ButtonDanger, ButtonNormal,
                                        ButtonPrimary, ComboBoxBase, FrameBase,
                                        FrameTabFooter, FrameTabHeader,
                                        LabelBase, SpacerHorizontal)
from anaconda_navigator.widgets.helperwidgets import LineEditSearch
from anaconda_navigator.widgets.manager.table import TableCondaPackages


# yapf: enable

_ = gettext.gettext


# --- Widgtes defined for CSS styling
# -----------------------------------------------------------------------------
class ComboBoxPackageFilter(ComboBoxBase):
    """Combobox used in CSS styling."""

    currentTextChanged = Signal(str)

    def __init__(self, *args, **kwargs):
        """Combobox used in CSS styling."""
        super(ComboBoxPackageFilter, self).__init__(*args, **kwargs)
        self.currentIndexChanged.connect(self._handle_index_changed)

    def _handle_index_changed(self, index):
        """Recreate signal not present in qt4."""
        if isinstance(index, int):
            self.currentTextChanged.emit(self.itemText(index))

    def setCurrentText(self, text):
        """Recreate method not present in qt4."""
        for i in range(self.count()):
            if text == self.itemText(i):
                self.setCurrentIndex(i)
                return


class ButtonPackageChannels(ButtonNormal):
    """Button used in CSS styling."""


class ButtonPackageOk(ButtonNormal):
    """Button used in CSS styling."""


class ButtonPackageApply(ButtonPrimary):
    """Button used in CSS styling."""


class ButtonPackageCancel(ButtonNormal):
    """Button used in CSS styling."""


class ButtonPackageUpdate(ButtonNormal):
    """Button used in CSS styling."""


class ButtonPackageClear(ButtonDanger):
    """Button used in CSS styling."""


class ProgressBarPackage(QProgressBar):
    """Progress bar used in CSS styling."""


class LabelPackageStatus(LabelBase):
    """Label used in CSS styling."""


class LabelPackageStatusAction(LabelBase):
    """Label used in CSS styling."""


class FramePackageTop(FrameBase):
    """Frame used in CSS styling."""


class FramePackageBottom(FrameBase):
    """Frame used in CSS styling."""

    pass


# --- Navigationr helper widgets
# -----------------------------------------------------------------------------
class FirstRowWidget(ButtonBase):
    """Widget located before table to handle focus in and tab focus."""

    sig_enter_first = Signal()

    def __init__(self, widget_before=None):
        """Widget located before table to handle focus in and tab focus."""
        QPushButton.__init__(self)
        self.widget_before = widget_before
        self.setAttribute(Qt.WA_LayoutUsesWidgetRect)  # Needed on mac

    @staticmethod
    def sizeHint():
        """Override Qt method."""
        return QSize(0, 0)

    def focusInEvent(self, event):
        """Override Qt method."""
        self.sig_enter_first.emit()

    def event(self, event):
        """Override Qt method."""
        if event.type() == QEvent.KeyPress:
            key = event.key()
            if key in [Qt.Key_Tab]:
                self.sig_enter_first.emit()
                return True
            else:
                return QPushButton.event(self, event)
        else:
            return QPushButton.event(self, event)


class LastRowWidget(ButtonBase):
    """Widget located after table to handle focus out and tab focus."""

    sig_enter_last = Signal()

    def __init__(self, widgets_after=None):
        """Widget located after table to handle focus out and tab focus."""
        QPushButton.__init__(self)
        self.widgets_after = widgets_after or []
        self.setAttribute(Qt.WA_LayoutUsesWidgetRect)  # Needed on mac
        self.setFocusPolicy(Qt.StrongFocus)

    def focusInEvent(self, event):
        """Override Qt method."""
        self.sig_enter_last.emit()

    def add_focus_widget(self, widget):
        """Add after focus widget."""
        if widget in self.widgets_after:
            return
        else:
            self.widgets_after.append(widget)

    def handle_tab(self):
        """Handle tab focus widget."""
        for w in self.widgets_after:
            if w.isVisible():
                w.setFocus()
                return
        self.setFocus()

    def event(self, event):
        """Override Qt method."""
        if event.type() == QEvent.KeyPress:
            key = event.key()
            if key in [Qt.Key_Backtab]:
                self.sig_enter_last.emit()
                return True
            else:
                return QPushButton.event(self, event)
        else:
            return QPushButton.event(self, event)

    @staticmethod
    def sizeHint():
        """Override Qt method."""
        return QSize(0, 0)


# --- Main widget
# -----------------------------------------------------------------------------
class CondaPackagesWidget(QWidget):
    """Conda Packages Widget."""

    sig_ready = Signal()
    sig_next_focus = Signal()

    # conda_packages_action_dict, pip_packages_action_dict
    sig_packages_action_requested = Signal(object, object)

    # button_widget, sender
    sig_channels_requested = Signal(object, object)

    # sender
    sig_update_index_requested = Signal(object)
    sig_cancel_requested = Signal(object)

    def __init__(self, parent, config=CONF):
        """Conda Packages Widget."""
        super(CondaPackagesWidget, self).__init__(parent)

        self._parent = parent
        self._current_model_index = None
        self._current_action_name = ''
        self._current_table_scroll = None
        self._hide_widgets = False

        self.api = AnacondaAPI()
        self.prefix = None

        self.style_sheet = None
        self.message = ''
        self.config = config

        self.tracker = GATracker()

        # Widgets
        self.bbox = QDialogButtonBox(Qt.Horizontal)
        self.button_cancel = ButtonPackageCancel('Cancel')
        self.button_channels = ButtonPackageChannels(_('Channels'))
        self.button_ok = ButtonPackageOk(_('Ok'))
        self.button_update = ButtonPackageUpdate(_('Update index...'))
        self.button_apply = ButtonPackageApply(_('Apply'))
        self.button_clear = ButtonPackageClear(_('Clear'))
        self.combobox_filter = ComboBoxPackageFilter(self)
        self.frame_top = FrameTabHeader()
        self.frame_bottom = FrameTabFooter()
        self.progress_bar = ProgressBarPackage(self)
        self.label_status = LabelPackageStatus(self)
        self.label_status_action = LabelPackageStatusAction(self)
        self.table = TableCondaPackages(self)
        self.textbox_search = LineEditSearch(self)
        self.widgets = [
            self.button_update, self.button_channels, self.combobox_filter, self.textbox_search, self.table,
            self.button_ok, self.button_apply, self.button_clear
        ]
        self.table_first_row = FirstRowWidget(widget_before=self.textbox_search)
        self.table_last_row = LastRowWidget(widgets_after=[
            self.button_apply,
            self.button_clear,
            self.button_cancel,
        ])

        # Widgets setup
        max_height = self.label_status.fontMetrics().height()
        max_width = self.textbox_search.fontMetrics().width('0' * 24)
        self.bbox.addButton(self.button_ok, QDialogButtonBox.ActionRole)
        self.button_ok.setMaximumSize(QSize(0, 0))
        self.button_ok.setVisible(False)
        self.button_channels.setCheckable(True)
        combo_items = [k for k in C.COMBOBOX_VALUES_ORDERED]
        self.combobox_filter.addItems(combo_items)
        self.combobox_filter.setMinimumWidth(120)
        self.progress_bar.setMaximumHeight(max_height * 1.2)
        self.progress_bar.setMaximumWidth(max_height * 12)
        self.progress_bar.setTextVisible(False)
        self.progress_bar.setVisible(False)
        self.setMinimumSize(QSize(480, 300))
        self.setWindowTitle(_("Conda Package Manager"))
        self.label_status.setFixedHeight(max_height * 1.5)
        self.textbox_search.setMaximumWidth(max_width)
        self.textbox_search.setMinimumWidth(max_width)
        self.textbox_search.setPlaceholderText('Search Packages')
        self.table_first_row.setMaximumHeight(0)
        self.table_last_row.setMaximumHeight(0)
        self.table_last_row.setVisible(False)
        self.table_first_row.setVisible(False)

        # Layout
        top_layout = QHBoxLayout()
        top_layout.addWidget(self.combobox_filter, 0, Qt.AlignCenter)
        top_layout.addWidget(SpacerHorizontal())
        top_layout.addWidget(self.button_channels, 0, Qt.AlignCenter)
        top_layout.addWidget(SpacerHorizontal())
        top_layout.addWidget(self.button_update, 0, Qt.AlignCenter)
        top_layout.addStretch()
        top_layout.addWidget(self.textbox_search, 0, Qt.AlignCenter)
        self.frame_top.setLayout(top_layout)

        middle_layout = QVBoxLayout()
        middle_layout.addWidget(self.table_first_row)
        middle_layout.addWidget(self.table)
        middle_layout.addWidget(self.table_last_row)

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.label_status_action)
        bottom_layout.addWidget(SpacerHorizontal())
        bottom_layout.addWidget(self.label_status)
        bottom_layout.addStretch()
        bottom_layout.addWidget(self.progress_bar)
        bottom_layout.addWidget(SpacerHorizontal())
        bottom_layout.addWidget(self.button_cancel)
        bottom_layout.addWidget(SpacerHorizontal())
        bottom_layout.addWidget(self.button_apply)
        bottom_layout.addWidget(SpacerHorizontal())
        bottom_layout.addWidget(self.button_clear)
        self.frame_bottom.setLayout(bottom_layout)

        layout = QVBoxLayout(self)
        layout.addWidget(self.frame_top)
        layout.addLayout(middle_layout)
        layout.addWidget(self.frame_bottom)
        self.setLayout(layout)

        # Signals and slots
        self.button_cancel.clicked.connect(lambda: self.sig_cancel_requested.emit(C.TAB_ENVIRONMENT))
        self.combobox_filter.currentTextChanged.connect(self.filter_package)
        self.button_apply.clicked.connect(self.apply_multiple_actions)
        self.button_clear.clicked.connect(self.clear_actions)
        self.button_channels.clicked.connect(self.show_channels)
        self.button_update.clicked.connect(self.update_package_index)
        self.textbox_search.textChanged.connect(self.search_package)
        self.table.sig_actions_updated.connect(self.update_actions)
        self.table.sig_status_updated.connect(self.update_status)

        self.table.sig_next_focus.connect(self.table_last_row.handle_tab)
        self.table.sig_previous_focus.connect(lambda: self.table_first_row.widget_before.setFocus())
        self.table_first_row.sig_enter_first.connect(self._handle_tab_focus)
        self.table_last_row.sig_enter_last.connect(self._handle_backtab_focus)

        self.button_cancel.setVisible(False)

    # --- Helpers
    # -------------------------------------------------------------------------
    def _handle_tab_focus(self):
        self.table.setFocus()
        if self.table.proxy_model:
            index = self.table.proxy_model.index(0, 0)
            self.table.setCurrentIndex(index)

    def _handle_backtab_focus(self):
        self.table.setFocus()
        if self.table.proxy_model:
            row = self.table.proxy_model.rowCount() - 1
            index = self.table.proxy_model.index(row, 0)
            self.table.setCurrentIndex(index)

    # --- Setup
    # -------------------------------------------------------------------------
    def setup(self, packages=None, model_data=None, prefix=None):
        """
        Setup packages.

        Populate the table with `packages` information.

        Parameters
        ----------
        packages: dict
            Grouped package information by package name.
        blacklist: list of str
            List of conda package names to be excluded from the actual package
            manager view.
        """
        self.table.setup_model(packages, model_data)
        combobox_text = self.combobox_filter.currentText()
        self.combobox_filter.setCurrentText(combobox_text)
        self.filter_package(combobox_text)
        self.table.setFocus()
        self.sig_ready.emit()

    # --- Other methods
    # -------------------------------------------------------------------------
    def apply_multiple_actions(self):
        """Apply multiple actions on packages."""
        logger.debug('')

        actions = self.table.get_actions()
        pip_actions = actions[C.PIP_PACKAGE]
        conda_actions = actions[C.CONDA_PACKAGE]

        self.sig_packages_action_requested.emit(conda_actions, pip_actions)

    def clear_actions(self):
        """Clear the table actions."""
        self.table.clear_actions()

    def filter_package(self, value):
        """Filter packages by type."""
        self.tracker.track_event('environments', 'filter_by', value)
        self.table.filter_status_changed(value)

    def search_package(self, text):
        """Search and filter packages by text."""
        self.table.search_string_changed(text)

    def show_channels(self):
        """Show channel dialog."""
        self.tracker.track_event('environments', 'click', 'Channels')
        self.sig_channels_requested.emit(
            self.button_channels,
            C.TAB_ENVIRONMENT,
        )

    def update_actions(self, number_of_actions):
        """Update visibility of buttons based on actions."""
        self.button_apply.setVisible(bool(number_of_actions))
        self.button_clear.setVisible(bool(number_of_actions))

    def update_package_index(self):
        """Update pacakge index."""
        self.tracker.track_event('environments', 'click', 'Update index...')
        self.sig_update_index_requested.emit(C.ENVIRONMENT_PACKAGE_MANAGER)

    # --- Common methods
    # -------------------------------------------------------------------------
    def ordered_widgets(self):
        pass

    def set_widgets_enabled(self, value):
        """Set the enabled status of widgets and subwidgets."""
        self.table.setEnabled(value)
        self.button_clear.setEnabled(value)
        self.button_apply.setEnabled(value)
        self.button_cancel.setEnabled(not value)
        self.button_cancel.setVisible(not value)

    def update_status(self, action='', message='', value=None, max_value=None):
        """
        Update status of package widget.

        - progress == None and max_value == None -> Not Visible
        - progress == 0 and max_value == 0 -> Busy
        - progress == n and max_value == m -> Progress values
        """
        # Elide if too big
        width = QApplication.desktop().availableGeometry().width()
        max_status_length = round(width * (1.0 / 2.0), 0)
        msg_percent = 0.70

        fm = self.label_status_action.fontMetrics()
        action = fm.elidedText(action, Qt.ElideRight, round(max_status_length * msg_percent, 0))
        message = fm.elidedText(message, Qt.ElideRight, round(max_status_length * (1 - msg_percent), 0))

        self.label_status_action.setText(action)
        self.label_status.setText(message)

        if max_value is None and value is None:
            self.progress_bar.setVisible(False)
        else:
            self.progress_bar.setVisible(True)
            self.progress_bar.setMaximum(max_value)
            self.progress_bar.setValue(value)

    def update_style_sheet(self, style_sheet=None):
        """Update custom CSS style sheet."""
        if style_sheet is None:
            self.style_sheet = load_style_sheet()
        else:
            self.style_sheet = style_sheet
        self.setStyleSheet(self.style_sheet)


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run conda packages widget test."""
    from anaconda_navigator.utils.qthelpers import qapplication
    app = qapplication()
    widget = CondaPackagesWidget(parent=None)
    widget.update_style_sheet()
    widget.show()
    sys.exit(app.exec_())


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Dialogs for removing environments."""

__all__ = ['RemoveDialog']

from qtpy import QtWidgets

from anaconda_navigator import widgets
from . import common


class RemoveDialog(common.EnvironmentActionsDialog):
    """Remove existing environment dialog."""

    def __init__(self, parent=None, name=None, prefix=None):
        """Remove existing environment `name` dialog."""
        super(RemoveDialog, self).__init__(parent=parent)

        # Widgets
        self.label_text = widgets.LabelBase('Do you want to remove the environment?')
        self.label_name = widgets.LabelBase('Name:')
        self.label_name_value = widgets.LabelBase(name)
        self.label_location = widgets.LabelBase('Location:')
        self.label_prefix = widgets.LabelBase(prefix)

        self.button_cancel = widgets.ButtonNormal('Cancel')
        self.button_ok = widgets.ButtonDanger('Remove')

        # Setup
        self.align_labels([self.label_name, self.label_location])
        self.label_prefix.setObjectName('environment-location')
        self.setWindowTitle('Remove environment')
        self.setMinimumWidth(380)
        self.label_name.setMinimumWidth(60)
        self.label_location.setMinimumWidth(60)

        # Layouts
        layout_name = QtWidgets.QHBoxLayout()
        layout_name.addWidget(self.label_name)
        layout_name.addWidget(widgets.SpacerHorizontal())
        layout_name.addWidget(self.label_name_value)
        layout_name.addStretch()

        layout_location = QtWidgets.QHBoxLayout()
        layout_location.addWidget(self.label_location)
        layout_location.addWidget(widgets.SpacerHorizontal())
        layout_location.addWidget(self.label_prefix)
        layout_location.addStretch()

        layout_buttons = QtWidgets.QHBoxLayout()
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_cancel)
        layout_buttons.addWidget(widgets.SpacerHorizontal())
        layout_buttons.addWidget(self.button_ok)

        layout = QtWidgets.QVBoxLayout()
        layout.addLayout(layout_name)
        layout.addWidget(widgets.SpacerVertical())
        layout.addLayout(layout_location)
        layout.addWidget(widgets.SpacerVertical())
        layout.addWidget(widgets.SpacerVertical())
        layout.addLayout(layout_buttons)
        self.setLayout(layout)

        # Signals
        self.button_ok.clicked.connect(self.accept)
        self.button_cancel.clicked.connect(self.reject)

        # Setup
        self.update_location()
        self.button_ok.setDisabled(False)



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Components for user accounts management."""

__all__ = ['AccountsComponent']

import typing
from urllib import parse
from qtpy import QtCore
from qtpy import QtWidgets
from conda_token import repo_config
from anaconda_navigator.api import nucleus
from anaconda_navigator import config
from anaconda_navigator import utils
from anaconda_navigator import widgets
from anaconda_navigator.widgets.dialogs import login as login_dialogs
from . import common

if typing.TYPE_CHECKING:
    import typing_extensions
    from anaconda_navigator.widgets import main_window
    from anaconda_navigator.widgets.dialogs.login import account_dialogs


class ButtonLabelLogin(QtWidgets.QLabel):
    """Button used in CSS styling."""


class ButtonLogin(widgets.ButtonPrimary):
    """Button used in CSS styling."""


class AccountsComponent(common.Component):
    """Component for account management."""

    __alias__ = 'accounts'

    def __init__(self, parent: 'main_window.MainWindow') -> None:
        """Initialize new :class:`~AccountsComponent` instance."""
        super().__init__(parent=parent)

        self.__authenticated: bool = False
        self.__brand: typing.Optional[str] = config.DEFAULT_BRAND
        self.__token: typing.Optional[str] = self.main_window.api._client_api.load_token()
        self.__url: typing.Optional[str] = None
        self.__username: str = ''

        self.__timer: 'typing_extensions.Final[QtCore.QTimer]' = QtCore.QTimer()
        self.__timer.setInterval(5000)
        self.__timer.timeout.connect(self.__check_for_new_login)

        self.__account_label: 'typing_extensions.Final[QtWidgets.QLabel]' = ButtonLabelLogin()
        self.__account_label.setFocusPolicy(QtCore.Qt.NoFocus)
        self.__account_label.setText('')
        self.__account_label.linkActivated.connect(lambda activated_url: self.main_window.open_url(activated_url))

        self.__login_button: 'typing_extensions.Final[QtWidgets.QPushButton]' = ButtonLogin()
        self.__login_button.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.__login_button.setDefault(True)
        self.__login_button.setText('Connect')
        self.__login_button.clicked.connect(self.__show_accounts)

    @property
    def account_label(self) -> QtWidgets.QLabel:
        """Label with details of account login."""
        return self.__account_label

    @property
    def login_button(self) -> QtWidgets.QPushButton:
        """Button to trigger login action."""
        return self.__login_button

    def update_login_status(self, user_data=None):
        """Update login button and information."""
        if self.main_window.config.get('main', 'logged_api_url') or user_data:
            self.__username = user_data.get('login', '') if user_data else self.__username
            self.__authenticated = True

        self.__update_account_label_text()

        # See: https://github.com/ContinuumIO/navigator/issues/1325
        self.main_window.api.client_reload()

        def apply_api_urls(worker, output, error):
            if output:
                self.__url = output.get('main_url', config.DEFAULT_ANACONDA_MAIN_URL)
                self.__brand = output.get('brand', config.DEFAULT_BRAND)
            else:
                self.__url = config.DEFAULT_ANACONDA_MAIN_URL
                self.__brand = config.DEFAULT_BRAND

            try:
                self.login_button.setEnabled(True)
            except RuntimeError:
                pass  # On CI: wrapped C/C++ object of type ButtonLinkLogin has been deleted

        self.login_button.setEnabled(False)
        worker = self.main_window.api.api_urls()
        worker.username = self.__username
        worker.sig_chain_finished.connect(apply_api_urls)
        QtWidgets.QApplication.restoreOverrideCursor()

    def __update_account_label_text(self) -> None:
        """Update login button and information."""
        result: typing.List[str] = []

        nucleus_account: typing.Optional[str] = nucleus.NucleusAPI().token.username
        if nucleus_account:
            result.append('Nucleus')

        if self.__authenticated:
            brand: str
            brand, _ = self.get_logged_api_label_url()
            result.append(brand)

        content: str = ''
        if result:
            content = 'Connected to ' + ', '.join(
                f'<a href="" style="color:#43B049;text-decoration:none">{item}</a>' for item in result
            )

        self.account_label.setText(content)
        self.account_label.setVisible(bool(content))

    def __show_accounts(self):
        """Open up login dialog or log out depending on logged status."""
        states: 'account_dialogs.AccountStateMapping' = {}

        nucleus_username: typing.Optional[str] = nucleus.NucleusAPI().token.username
        if nucleus_username:
            states['nucleus'] = login_dialogs.AccountState(
                status=login_dialogs.AccountStatus.ACTIVE,
                username=nucleus_username,
            )
        else:
            states['nucleus'] = login_dialogs.AccountState(status=login_dialogs.AccountStatus.AVAILABLE)

        if self.__authenticated:
            brand, _ = self.get_logged_api_label_url()
            if brand == config.ANACONDA_ORG_BRAND:
                states['individual'] = login_dialogs.AccountState(
                    status=login_dialogs.AccountStatus.ACTIVE,
                    username=self.__username,
                )
            elif brand == config.COMMERCIAL_EDITION_BRAND:
                states['commercial'] = login_dialogs.AccountState(
                    status=login_dialogs.AccountStatus.ACTIVE,
                    username=self.__username,
                )
            elif brand == config.TEAM_EDITION_BRAND:
                states['team'] = login_dialogs.AccountState(
                    status=login_dialogs.AccountStatus.ACTIVE,
                    username=self.__username,
                )
            elif brand == config.ENTERPRISE_EDITION_BRAND:
                states['enterprise'] = login_dialogs.AccountState(
                    status=login_dialogs.AccountStatus.ACTIVE,
                    username=self.__username,
                )
        else:
            states['individual'] = login_dialogs.AccountState(status=login_dialogs.AccountStatus.AVAILABLE)
            states['commercial'] = login_dialogs.AccountState(status=login_dialogs.AccountStatus.AVAILABLE)
            states['team'] = login_dialogs.AccountState(status=login_dialogs.AccountStatus.AVAILABLE)
            states['enterprise'] = login_dialogs.AccountState(status=login_dialogs.AccountStatus.AVAILABLE)

        selector = login_dialogs.AccountsDialog(
            parent=self.main_window,
            anchor=self.login_button,
            states=states,
        )
        selector.sig_accepted.connect(self.__process_accounts)
        selector.show()

    def __process_accounts(self, outcome: login_dialogs.AccountOutcome, value: login_dialogs.AccountValue) -> None:
        """"""
        if outcome == login_dialogs.AccountOutcome.REJECT:
            return

        if outcome == login_dialogs.AccountOutcome.LOGIN_REQUEST:
            {
                login_dialogs.AccountValue.NUCLEUS: self.log_into_nucleus,
                login_dialogs.AccountValue.INDIVIDUAL_EDITION: self.log_into_individual_edition,
                login_dialogs.AccountValue.COMMERCIAL_EDITION: self.log_into_commercial_edition,
                login_dialogs.AccountValue.TEAM_EDITION: self.log_into_team_edition,
                login_dialogs.AccountValue.ENTERPRISE_EDITION: self.log_into_enterprise_edition,
            }[value]()
            return

        if outcome == login_dialogs.AccountOutcome.LOGOUT_REQUEST:
            {
                login_dialogs.AccountValue.NUCLEUS: self.log_out_from_nucleus,
                login_dialogs.AccountValue.INDIVIDUAL_EDITION: self.log_out_from_repository,
                login_dialogs.AccountValue.COMMERCIAL_EDITION: self.log_out_from_repository,
                login_dialogs.AccountValue.TEAM_EDITION: self.log_out_from_repository,
                login_dialogs.AccountValue.ENTERPRISE_EDITION: self.log_out_from_repository,
            }[value]()
            return

        raise ValueError('Unexpected login outcome')

    def log_into_nucleus(self) -> None:
        """Open dialogs to log into Anaconda Nucleus."""
        if login_dialogs.NucleusLoginPage(parent=self.main_window).exec_():
            self.__update_account_label_text()

    def log_into_individual_edition(self) -> None:
        """Open dialogs to log into Anaconda Individual Edition."""
        credentials_dialog: 'typing_extensions.Final[QtWidgets.QDialog]' = login_dialogs.AnacondaLoginPage(
            parent=self.main_window,
        )
        credentials_dialog.exec_()
        self.__postprocess_dialog(dialog=credentials_dialog)

    def log_into_commercial_edition(self) -> None:
        """Open dialogs to log into Anaconda Commercial Edition."""
        credentials_dialog: 'typing_extensions.Final[QtWidgets.QDialog]' = login_dialogs.CommercialEditionLoginPage(
            parent=self.main_window,
        )
        credentials_dialog.exec_()
        self.__postprocess_dialog(dialog=credentials_dialog)

    def log_into_team_edition(self) -> None:
        """Open dialogs to log into Anaconda Team Edition."""
        if not self.main_window.config.get('main', 'team_edition_api_url'):
            domain_dialog: 'typing_extensions.Final[QtWidgets.QDialog]' = login_dialogs.TeamEditionSetDomainPage(
                parent=self.main_window,
            )
            if not domain_dialog.exec_():
                return

        credentials_dialog: 'typing_extensions.Final[QtWidgets.QDialog]' = login_dialogs.TeamEditionLoginPage(
            parent=self.main_window,
        )
        if not credentials_dialog.exec_():
            return

        login_dialogs.TeamEditionAddChannelsPage(parent=self.main_window).exec_()
        self.__postprocess_dialog(credentials_dialog)

    def log_into_enterprise_edition(self) -> None:
        """Open dialogs to log into Anaconda Enterprise Edition."""
        if not self.main_window.config.get('main', 'enterprise_4_repo_api_url'):
            domain_dialog: 'typing_extensions.Final[QtWidgets.QDialog]' = login_dialogs.EnterpriseRepoSetDomainPage(
                parent=self.main_window,
            )
            if not domain_dialog.exec_():
                return

        credentials_dialog: 'typing_extensions.Final[QtWidgets.QDialog]' = login_dialogs.EnterpriseRepoLoginPage(
            parent=self.main_window,
        )
        if not credentials_dialog.exec_():
            return

        login_dialogs.NoticePage(parent=self.main_window).exec_()
        self.__postprocess_dialog(credentials_dialog)

    def log_out_from_nucleus(self) -> None:
        """Log out from Anaconda Nucleus."""
        nucleus.NucleusAPI().logout()
        self.__update_account_label_text()

    def log_out_from_repository(self) -> None:
        """Log out from all repositories (Individual, Commercial, Team and Enterprise editions)."""
        QtWidgets.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
        repo_config.token_remove()
        self.main_window.api.remove_login_data()
        self.main_window.api.logout()

        self.__authenticated = False
        self.__token = None
        self.__username = ''

        self.main_window.sig_logged_out.emit()
        self.main_window.tracker.track_event('authenticate', 'logout', label=self.__username)
        self.update_login_status()

    def __postprocess_dialog(self, dialog: QtWidgets.QDialog) -> None:
        """Apply changes from the login dialog."""
        if dialog.result():
            self.__authenticated = True
            self.__username = dialog.username
            self.main_window.api.detect_commercial_edition_login()
            self.main_window.sig_logged_in.emit()
            if self.main_window.tracker:
                self.main_window.tracker.track_event('authenticate', 'login', label=self.__username)
        self.main_window._track_tab()
        if dialog.result():
            self.update_login_status()

    def __check_for_new_login(self) -> None:
        """
        Check for new login status periodically on the system.

        Also checks for internet connectivity and updates.
        """
        new_token: typing.Optional[str] = self.main_window.api._client_api.load_token()
        if new_token != self.__token:
            self.__token = new_token
            if new_token is None:
                self.log_out_from_repository()
            else:
                pass  # TODO: How to relogin if logged from command line??

    def get_logged_api_label_url(self) -> typing.Tuple[str, str]:
        logged_api_url, anaconda_api_url, enterprise_4_repo_api_url, team_edition_api_url, commercial_edition_url = (
            parse.urlsplit(self.main_window.config.get('main', config_option) or '') for config_option in (
                'logged_api_url', 'anaconda_api_url', 'enterprise_4_repo_api_url', 'team_edition_api_url',
                'commercial_edition_url'
            )
        )
        brand: str = self.main_window.config.get('main', 'logged_brand') or config.ANACONDA_ORG_BRAND
        domain: str = utils.get_domain_from_api_url(anaconda_api_url)
        if logged_api_url.netloc:
            if logged_api_url == enterprise_4_repo_api_url:
                domain = utils.get_domain_from_api_url(enterprise_4_repo_api_url)

            elif logged_api_url == team_edition_api_url:
                domain = parse.urljoin(team_edition_api_url.geturl(), '/channels/')

            elif logged_api_url == commercial_edition_url:
                domain = commercial_edition_url.geturl()

        return brand, domain

    def start_timers(self) -> None:
        """Start component timers."""
        self.__timer.start()

    def stop_timers(self) -> None:
        """Stop component timers."""
        self.__timer.stop()



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Utils for login dialogs."""

# from collections import namedtuple
import typing
from qtpy.QtCore import QRegExp

EMAIL_RE = QRegExp(r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$')
USER_RE = QRegExp(r'^[A-Za-z0-9_\.][A-Za-z0-9_\.\-]+$')
FORGOT_LOGIN_URL = 'account/forgot_username'
FORGOT_PASSWORD_URL = 'account/forgot_password'


class TextContainer(typing.NamedTuple):

    title: typing.Optional[str] = None
    header_frame_logo_path: typing.Optional[str] = None
    info_frame_text: typing.Optional[str] = None
    info_frame_logo_path: typing.Optional[str] = None
    form_forgot_links_msg: typing.Optional[str] = None
    form_primary_text: typing.Optional[str] = None
    form_secondary_text: typing.Optional[str] = None
    form_input_label_text: typing.Optional[str] = None
    form_submit_button_text: typing.Optional[str] = None
    message_box_error_text: typing.Optional[str] = None



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Common components for environment dialogs."""

__all__ = ['LineEditEnvironment', 'EnvironmentActionsDialog']

import os
import typing
from qtpy import QtCore
from qtpy import QtGui
from qtpy import QtWidgets
from anaconda_navigator import widgets
from anaconda_navigator.widgets import common as global_commons
from anaconda_navigator.widgets import dialogs

if typing.TYPE_CHECKING:
    import typing_extensions


class LineEditEnvironment(widgets.LineEditBase):
    """
    Custom line edit to handle regex for naming an environment.
    """

    VALID_RE = QtCore.QRegExp('^[A-Za-z][A-Za-z0-9_ -]{0,30}$')

    sig_return_pressed = QtCore.Signal()
    sig_escape_pressed = QtCore.Signal()
    sig_copied = QtCore.Signal()

    def __init__(self, *args, **kwargs):
        """Custom line edit for naming an environment."""
        super(LineEditEnvironment, self).__init__(*args, **kwargs)
        self._validator = QtGui.QRegExpValidator(self.VALID_RE)
        self.menu = QtWidgets.QMenu(parent=self)
        self.setValidator(self._validator)
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

    def event(self, event):
        """Override Qt method."""
        if (
                event.type() == QtCore.QEvent.MouseButtonPress
                and
                event.buttons() & QtCore.Qt.RightButton
                and
                not self.isEnabled()
        ):
            self.show_menu(event.pos())
            return True
        else:
            return super(LineEditEnvironment, self).event(event)

    def keyPressEvent(self, event):
        """Override Qt method."""
        key = event.key()
        # Display a copy menu in case the widget is disabled.
        if event.matches(QtGui.QKeySequence.Paste):
            clipboard = QtWidgets.QApplication.clipboard()
            text = clipboard.text()
            if self.VALID_RE.exactMatch(text):
                self.setText(text)
                return
        else:
            if key in [QtCore.Qt.Key_Return, QtCore.Qt.Key_Enter]:
                self.sig_return_pressed.emit()
            elif key in [QtCore.Qt.Key_Escape]:
                self.sig_escape_pressed.emit()
        super(LineEditEnvironment, self).keyPressEvent(event)

    def show_menu(self, pos):
        """Show copy menu for channel item."""
        self.menu.clear()
        copy = QtWidgets.QAction("&Copy", self.menu)
        copy.triggered.connect(self.copy_text)
        self.menu.addAction(copy)
        self.menu.setEnabled(True)
        self.menu.exec_(self.mapToGlobal(pos))

    def copy_text(self):
        """Copy environment text to clipboard."""
        clipboard = QtWidgets.QApplication.clipboard()
        clipboard.setText(self.text())
        self.sig_copied.emit()


class EnvironmentActionsDialog(dialogs.DialogBase):
    """Base dialog with common methods for all dialogs."""

    BASE_DIALOG_WIDTH = 480

    sig_setup_ready = QtCore.Signal()

    def __init__(self, parent=None, api=None):
        """Base dialog with common methods for all dialogs."""
        super(EnvironmentActionsDialog, self).__init__(parent=parent)

        self.info = None
        self._packages = None
        self.envs_dirs = None
        self.environments = None
        self.api = api
        self.channels = None

        # Widgets to be defined on subclass __init__
        self.text_name = None
        self.label_prefix = None

    def setup(self, worker=None, conda_data=None, error=None):
        """Setup the dialog conda information as a result of a conda worker."""
        if conda_data:
            conda_info = conda_data.get('processed_info', {})
            self.info = conda_info
            self._packages = conda_data.get('packages')
            self.envs_dirs = conda_info['__envs_dirs_writable']
            self.environments = conda_info['__environments']
            self.channels = conda_info.get('channels')
            self.refresh()
            self.sig_setup_ready.emit()

    def update_location(self):
        """Update the location (prefix) text."""
        self.button_ok.setDisabled(True)
        if self.environments:
            fm = self.label_prefix.fontMetrics()
            prefix = fm.elidedText(self.prefix, QtCore.Qt.ElideLeft, 300)
            self.label_prefix.setText(prefix)
            self.label_prefix.setToolTip(self.prefix)

    def refresh(self):
        """Update the status of buttons based data entered."""
        raise NotImplementedError

    def is_valid_env_name(self, env_name):
        """
        Check that an environment has a valid name.
        On Windows is case insensitive.
        """
        env_names = self.environments.values() if self.environments else []
        if os.name == 'nt':  # pragma: no cover unix
            env_name = env_name.lower()
            env_names = [e.lower() for e in env_names]

        return bool(env_name) and (env_name not in env_names)

    @staticmethod
    def align_labels(label_widgets):
        """Align label widgets to the right."""
        for widget in label_widgets:
            widget.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

    @property
    def name(self):  # noqa: D401
        """Return the content without extra spaces for the name of the env."""
        text = ''
        if self.text_name:
            text = self.text_name.text().strip()
        return text

    @property
    def prefix(self):  # noqa: D401
        """Return the full prefix (location) as entered in the dialog."""
        result: str
        if self.envs_dirs and self.name:
            result = os.path.join(self.envs_dirs[0], self.name)
        else:
            result = self.name
        return result


class OpenIconButton(global_commons.IconButton):
    """Button with "open" icon."""


class WarningIcon(QtWidgets.QLabel):
    """
    Icon with exclamation sign.

    Used by :class:`~WarningLabel` and its derivatives.
    """


class WarningLabel(QtWidgets.QFrame):
    """
    Combined label with warning icon at the beginning.

    Icon is shown only when the main text is not empty.
    """

    def __init__(self, text: str = '') -> None:
        """Initialize new :class:`~WarningLabel` instance."""
        super().__init__()

        self.__icon: 'typing_extensions.Final[WarningIcon]' = WarningIcon()
        self.__icon.setVisible(bool(text))

        self.__content: 'typing_extensions.Final[widgets.LabelBase]' = widgets.LabelBase()
        self.__content.setText(text)
        self.__content.setWordWrap(True)

        self.__layout: 'typing_extensions.Final[QtWidgets.QHBoxLayout]' = QtWidgets.QHBoxLayout()
        self.__layout.addWidget(self.__icon, 0, QtCore.Qt.AlignTop)
        self.__layout.addWidget(self.__content, 1, QtCore.Qt.AlignTop)
        self.__layout.setContentsMargins(0, 0, 0, 0)
        self.__layout.setSpacing(4)

        self.setLayout(self.__layout)

    @property
    def text(self) -> str:  # noqa: D401
        """Content of the main label."""
        return self.__content.text()

    @text.setter
    def text(self, value: str) -> None:
        """Update `text` value."""
        self.__content.setText(value)
        self.__icon.setVisible(bool(value))


class WarningBlock(WarningLabel):
    """Same as :class:`~WarningLabel`, but with colored background."""

    def __init__(self, text: str = '') -> None:
        """Initialize new :class:`~WarningBlock` instance."""
        super().__init__(text=text)



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Dialogs for creating new environments."""

__all__ = ['CreateDialog']

import collections
import re
import sys
import typing

from qtpy import QtCore
from qtpy import QtWidgets

from anaconda_navigator.api import conda_api
from anaconda_navigator import utils
from anaconda_navigator import widgets
from . import common


class PackageGroupOption(typing.NamedTuple):
    """
    Details about single option in package group (e.g. single version of `python` or `r` package).

    :param title: Title of the option, which is displayed in the combobox (e.g. '3.8.11').
    :param packages: List of packages, that should be installed when this option is selected
                     (e.g. ['python=3.8', 'extra-python-package'])

    Three values are used to sort available options in combobox: (`sort_prefix`, `sort_version`, `sort_postfix`).
    `sort_prefix` and `sort_postfix` are sorted lexicographically, while `sort_version` - by version order.
    """

    title: str
    packages: typing.Tuple[str, ...] = ()
    sort_prefix: str = ''
    sort_version: str = ''
    sort_postfix: str = ''


def compare_options(first: str, second: str) -> typing.Tuple[int, int]:
    """
    Get the difference between `first` and `second` lines.

    :returns: Tuple of:

              - Length of the common part of both strings (from the beginning)
              - Total length of mismatched tails of both strings (with a negative sign)
    """
    result: int = 0

    left: str
    right: str
    for left, right in zip(first, second):
        if left == right:
            result += 1
        else:
            break

    if result == 0:
        return 0, 0

    return result, 2 * result - len(first) - len(second)


class PackageGroup(QtCore.QObject):
    """
    Controls for single package group (`python`, `r`).

    :param title: Name of the group to show in interface.
    :param enabled: Enable this group (check the checkbox) as soon, as any option will be pushed to the group.
    :param default_option: Default option, that should be selected.

                           This value might be an "approximate" one - option that will be closest to it will be
                           selected. E.g. `default_option`="3.8" may be used to select "3.8.11" and not "3.7.11" or
                           "3.9.5".

                           If no value will be provided - last option will be selected (corresponds to "latest").
    """

    sig_refreshed = QtCore.Signal()

    def __init__(
            self,
            title: str,
            enabled: bool = False,
            default_option: typing.Optional[str] = None,
    ) -> None:
        """Initialize new :class:`~PackageGroup` instance."""
        super().__init__()

        self.enabled = enabled
        self.default_option = default_option

        self.checkbox = widgets.CheckBoxBase(title)
        self.checkbox.setChecked(False)
        self.checkbox.setDisabled(True)
        self.checkbox.stateChanged.connect(self.refresh)

        self.combo = widgets.ComboBoxBase()
        self.combo.setDisabled(True)

    @property
    def checked(self) -> bool:
        """Checkbox is checked for this group."""
        return self.checkbox.isChecked()

    @property
    def current(self) -> typing.Sequence[str]:
        """List of packages, that should be installed."""
        if not self.checkbox.isChecked():
            return ()

        data: typing.Optional[PackageGroupOption] = self.combo.currentData()
        if data is None:
            return ()
        return data.packages

    def push(self, options: typing.Iterable[PackageGroupOption]) -> None:
        """
        Add new `options` to the group.

        These options will be merged with the existing ones.
        """
        total: typing.Set[PackageGroupOption] = set()

        # Add existing options
        index: int = 0
        count: int = self.combo.count()
        while index < count:
            total.add(self.combo.itemData(index))
            index += 1

        # Merge with new options
        total.update(options)

        # Sort all options
        versions_order: typing.Mapping[str, int] = {
            item: index
            for index, item in enumerate(utils.sort_versions([item.sort_version for item in total]))
        }
        ordered: typing.Sequence[PackageGroupOption] = sorted(
            total,
            key=lambda item: (item.sort_prefix, versions_order[item.sort_version], item.sort_postfix),
        )

        # Push options
        ideal: str = self.combo.currentText() or self.default_option or ''

        best_value: str = ''
        best_score: typing.Tuple[int, int] = (0, 0)

        seen: typing.Set[str] = set()

        option: PackageGroupOption
        self.combo.clear()
        for index, option in enumerate(ordered):
            if option.title in seen:
                continue
            self.combo.addItem(option.title, option)
            seen.add(option.title)

            # Find the best matching option from the available ones
            # This part supports "approximate" default_options (e.g. default_option="3.8" may match "3.8.11")
            current_score: typing.Tuple[int, int] = compare_options(ideal, option.title)
            if current_score > best_score:
                best_value = option.title
                best_score = current_score

        if best_value:
            self.combo.setCurrentText(best_value)
        elif self.combo.count() > 0:
            self.combo.setCurrentIndex(self.combo.count() - 1)

        # Triggers
        self.refresh()

    def refresh(self) -> None:
        """Update state of the controls of the group."""
        if self.combo.count() == 0:
            self.checkbox.setEnabled(False)
            self.checkbox.setChecked(False)
        elif not self.checkbox.isEnabled():
            self.checkbox.setEnabled(True)
            self.checkbox.setChecked(self.enabled)

        self.combo.setEnabled(self.checkbox.isChecked())

        self.sig_refreshed.emit()


class CreateDialog(common.EnvironmentActionsDialog):
    """Create new environment dialog."""

    sig_environments_updated = QtCore.Signal(object, object)
    sig_setup_ready = QtCore.Signal()
    sig_check_ready = QtCore.Signal()

    def __init__(self, parent=None, api=None):
        """Create new environment dialog."""
        super(CreateDialog, self).__init__(parent=parent, api=api)

        self.setMinimumWidth(self.BASE_DIALOG_WIDTH)
        self.setWindowTitle("Create new environment")

        self.label_name = widgets.LabelBase("Name:")
        self.label_name.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        self.text_name = common.LineEditEnvironment()
        self.text_name.setPlaceholderText("New environment name")
        self.text_name.textChanged.connect(self.refresh_name)

        self.label_location = widgets.LabelBase("Location:")
        self.label_location.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        self.label_prefix = widgets.LabelBase('')
        self.label_prefix.setObjectName('environment-location')

        self.label_packages = widgets.LabelBase("Packages:")
        self.label_packages.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)

        self.button_ok = widgets.ButtonPrimary('Create')
        self.button_ok.clicked.connect(self.accept)
        self.button_ok.setDisabled(True)

        self.button_cancel = widgets.ButtonNormal('Cancel')
        self.button_cancel.clicked.connect(self.reject)

        package_group: PackageGroup
        self.package_groups: typing.Mapping[str, PackageGroup] = collections.OrderedDict((
            (
                'python',
                PackageGroup(
                    title='Python',
                    enabled=True,
                    default_option=f'{sys.version_info.major}.{sys.version_info.minor}',
                ),
            ),
            ('r', PackageGroup('R')),
        ))
        for package_group in self.package_groups.values():
            package_group.sig_refreshed.connect(self.refresh)

        index: int
        layout_packages = QtWidgets.QGridLayout()
        for index, package_group in enumerate(self.package_groups.values()):
            if index:
                layout_packages.addWidget(widgets.SpacerVertical(), 2 * index - 1, 0)
            layout_packages.addWidget(package_group.checkbox, 2 * index, 0)
            layout_packages.addWidget(widgets.SpacerHorizontal(), 2 * index, 1)
            layout_packages.addWidget(package_group.combo, 2 * index, 2)

        grid = QtWidgets.QGridLayout()
        grid.addWidget(self.label_name, 0, 0, 1, 1)
        grid.addWidget(widgets.SpacerHorizontal(), 0, 1, 1, 1)
        grid.addWidget(self.text_name, 0, 2, 1, 4)
        grid.addWidget(widgets.SpacerVertical(), 1, 0, 1, 1)
        grid.addWidget(self.label_location, 2, 0, 1, 1)
        grid.addWidget(widgets.SpacerHorizontal(), 2, 1, 1, 1)
        grid.addWidget(self.label_prefix, 2, 2, 1, 4)
        grid.addWidget(widgets.SpacerVertical(), 3, 0, 1, 1)
        grid.addWidget(self.label_packages, 4, 0, 1, 1)
        grid.addWidget(widgets.SpacerHorizontal(), 4, 1, 1, 1)
        grid.addLayout(layout_packages, 4, 2, 3, 1)

        layout_buttons = QtWidgets.QHBoxLayout()
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_cancel)
        layout_buttons.addWidget(widgets.SpacerHorizontal())
        layout_buttons.addWidget(self.button_ok)

        main_layout = QtWidgets.QVBoxLayout()
        main_layout.addLayout(grid)
        main_layout.addWidget(widgets.SpacerVertical())
        main_layout.addWidget(widgets.SpacerVertical())
        main_layout.addLayout(layout_buttons)

        self.setLayout(main_layout)
        self.text_name.setFocus()

        self.sig_setup_ready.connect(self._request_python)
        self.sig_setup_ready.connect(self._request_r)

    def _request_package(
            self,
            name: str,
            handler: typing.Callable[[typing.Any, typing.Any, typing.Any], None],
    ) -> None:
        """
        Request search of available package distributions.

        It tries to search in local cache first.
        If nothing found in cache - conda search might be used to find more in current channels.

        :param name: Name of the package to request versions for.
        :param handler: Handler to process search results with.
        """
        cached: typing.List[str] = self._packages.get(name, {}).get('versions', [])
        if cached:
            handler(None, {name: [{'version': item} for item in cached]}, '')
        else:
            conda_api.CondaAPI().search(name, offline=self.api.is_offline()).sig_finished.connect(handler)

    def _request_python(self) -> None:
        """Request list of available python versions from Conda."""
        self._request_package('python', self._fetch_python)

    def _fetch_python(self, _worker, output, error) -> None:
        """Parse available python versions from Conda."""
        if error:
            return

        pattern: typing.Pattern[str] = re.compile(r'^\d+\.\d+(?!\d)')

        version: str
        versions: typing.Dict[str] = {}
        raw_versions: typing.Sequence[str] = utils.sort_versions(
            versions={item['version'] for item in output.get('python', [])},
        )
        for version in raw_versions:
            reg_match: typing.Optional[typing.Match[str]] = pattern.match(version)
            if reg_match:
                versions[reg_match.group(0)] = version

        self.package_groups['python'].push([
            PackageGroupOption(long, (f'python={short}',), sort_prefix='python', sort_version=long)
            for short, long in versions.items()
        ])

    def _request_r(self) -> None:
        """Request list of available r versions from Conda."""
        self._request_package('r-base', self._fetch_r)

    def _fetch_r(self, _worker, output, error) -> None:
        """Parse available r versions from Conda."""
        if error:
            return

        versions: typing.Set[str] = {item['version'] for item in output.get('r-base', [])}

        self.package_groups['r'].push([
            PackageGroupOption(item, (f'r-base={item}', 'r-essentials'), sort_prefix='r-base', sort_version=item)
            for item in versions
        ])

    def refresh_name(self) -> None:
        """Update information, that is calculated from the name."""
        self.update_location()
        self.refresh()

    def refresh(self) -> None:
        """Update status of buttons based on data."""
        valid_environments: bool = bool(self.environments)
        valid_name: bool = self.is_valid_env_name(self.name)
        valid_package: bool = any(item.checked for item in self.package_groups.values())
        self.button_ok.setEnabled(valid_environments and valid_name and valid_package)

    @property
    def packages(self) -> typing.Sequence[str]:
        """List of packages, that must be installed in the new environment."""
        result: typing.Set[str] = set()

        package_group: PackageGroup
        for package_group in self.package_groups.values():
            result.update(package_group.current)

        return list(result)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Logger utilties."""

# yapf: disable

# Standard library imports
import json
import logging.handlers
import os

# Local imports
from anaconda_navigator.config import (LOG_FILENAME, LOG_FOLDER,
                                       MAX_LOG_FILE_SIZE)


# yapf: disable

# Constants
JSON_SEPARATOR = '\x00'

# Main logger instance
logger = logging.getLogger('navigator')


class ContextFilter(logging.Filter):
    """This is a filter that replaces `\` by `/` for json parsing."""

    @staticmethod
    def filter(record):
        """On windows make sure that slashes do not interfere with json."""
        # print([i for i in dir(record) if not i.startswith('_')])
        if isinstance(record.msg, dict):
            for k, v in record.msg.items():
                if isinstance(v, str):
                    record.msg[k] = v.replace('\\', '/')
        elif record.msg and isinstance(record.msg, str):
            record.msg = record.msg.replace('\\', '/')
            record.msg = repr(record.msg).replace('"', "'")

            if record.msg[0] == "'":
                record.msg = record.msg[1:]
            if record.msg[-1] == "'":
                record.msg = record.msg[:-1]

        if record.pathname:
            record.pathname = record.pathname.replace('\\', '/')

        return True


def setup_logger(
    log_level=logging.WARNING,
    log_folder=LOG_FOLDER,
    log_filename=LOG_FILENAME,
    log_file_size=MAX_LOG_FILE_SIZE,
    log_backup_count=5
):
    """Setup, create, and set logger."""
    global logger

    if not os.path.isdir(log_folder):
        os.makedirs(log_folder)

    log_file_path = os.path.join(log_folder, log_filename)

    # Reset extra loggers
    root_logger = logging.getLogger()
    root_logger.handlers = []

    # Setup logger for navigator application and reset handlers
    logger.setLevel(logging.DEBUG)
    logger.handlers = []

    # Create file handler
    file_handler = logging.handlers.RotatingFileHandler(
        log_file_path,
        maxBytes=log_file_size,
        backupCount=log_backup_count,
    )

    file_handler.setLevel(log_level)

    # Create console handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(log_level)

    # Create formatters
    file_formatter = logging.Formatter(
        '{'
        '"time": "%(asctime)s", '
        '"level": "%(levelname)s", '
        '"module": "%(module)s", '
        '"method": "%(funcName)s", '
        '"line": %(lineno)d, '
        '"path": "%(pathname)s", '
        '"message": "%(message)s"'
        '}' + JSON_SEPARATOR,
    )
    console_formatter = logging.Formatter(
        '%(asctime)s - %(levelname)s %(module)s.%(funcName)s:%(lineno)d\n'
        '%(message)s\n',
    )

    # Set formatters
    file_handler.setFormatter(file_formatter)
    console_handler.setFormatter(console_formatter)

    # Add filters
    filt = ContextFilter()
    file_handler.addFilter(filt)
    console_handler.addFilter(filt)

    # Add handlers
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)
    logger.info('Setting up logger')


def log_files(log_folder=LOG_FOLDER, log_filename=LOG_FILENAME):
    """
    Return all available log files located inside the logs folder.

    Files starting with a `.` are ignored as well as files not including the
    `log_filename` as part of the name.
    """
    paths = []
    if os.path.isdir(log_folder):
        log_files = os.listdir(log_folder)
        for log_file in sorted(log_files):
            log_file_path = os.path.join(log_folder, log_file)
            if (os.path.isfile(log_file_path) and log_filename in log_file and
                    not log_file.startswith('.')):
                paths.append(log_file_path)
    return paths


def clean_logs(log_folder=LOG_FOLDER):
    """Remove logs in old plain text format."""
    for path in log_files(log_folder):
        if os.path.isfile(path):
            with open(path, 'r') as f:
                data = f.read()

            new_lines = []
            lines = data.split('\n')
            for line in lines:
                if JSON_SEPARATOR in line:
                    new_lines.append(line)

            new_separator = JSON_SEPARATOR + '\n'
            with open(path, 'w') as f:
                f.write(new_separator.join(new_lines))


def load_log(log_file_path):
    """Load log file and return list of items."""
    if os.path.isfile(log_file_path):
        with open(log_file_path, 'r') as f:
            data = f.read()
        json_lines = data.split(JSON_SEPARATOR)

    # Remove empty lines
    json_lines = [line for line in json_lines if line and line != '\n']

    data = []
    for i, line in enumerate(json_lines):
        try:
            data.append(json.loads(line))
        except Exception as e:
            logger.warning('Line {0}. Exception - {1}'.format(i, str(e)))
    return data



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for logger window."""

# yapf: disable

# Standard library imports
import logging
import os

# Third party imports
from qtpy.QtCore import Qt
import pytest

# Local imports
from anaconda_navigator.utils.fixtures import tmpfile
from anaconda_navigator.utils.logs import logger, setup_logger
from anaconda_navigator.widgets.dialogs.logger import LogViewerDialog


# yapf: enable

# Constants
TEST_TEXT = 'WARNING TEST MESSAGE'
TEST_TEXT_EXTRA = 'SOMETHING ELSE'
tmpfile


# --- Fixtures
# -----------------------------------------------------------------------------
@pytest.fixture()
def loggerbot(qtbot, tmpfile):
    temp_log_folder = os.path.dirname(tmpfile)
    temp_log_filename = os.path.basename(tmpfile + '.log')

    setup_logger(log_level=logging.WARNING, log_folder=temp_log_folder, log_filename=temp_log_filename)
    logger.warning(TEST_TEXT)
    logger.warning(TEST_TEXT_EXTRA)
    widget = LogViewerDialog(log_folder=temp_log_folder, log_filename=temp_log_filename)
    widget.show()
    qtbot.addWidget(widget)
    return qtbot, widget


# --- Tests
# -----------------------------------------------------------------------------
def test_simple(loggerbot):
    qtbot, widget = loggerbot
    data = widget.row_data(0)
    message = data.get('message')

    assert widget.table_logs.rowCount() > 0
    assert TEST_TEXT in message


def test_search(loggerbot):
    qtbot, widget = loggerbot
    qtbot.keyClicks(widget.text_search, TEST_TEXT)

    assert widget.table_logs.isRowHidden(1)

    qtbot.keyClicks(widget.text_search, 'spam!')

    assert widget.table_logs.isRowHidden(0)
    assert widget.table_logs.isRowHidden(1)


def test_copy(loggerbot):
    from qtpy.QtWidgets import QApplication
    app = QApplication.instance()
    qtbot, widget = loggerbot
    qtbot.mouseClick(widget.button_copy, Qt.LeftButton)
    clipped = app.clipboard().text()

    assert TEST_TEXT in clipped



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Dialogs for cloning environments."""

__all__ = ['CloneDialog']

from qtpy import QtWidgets  # QGridLayout, QHBoxLayout, QVBoxLayout

from anaconda_navigator import widgets
from . import common


class CloneDialog(common.EnvironmentActionsDialog):
    """Clone environment dialog."""

    def __init__(self, parent=None, clone_from_name=None):
        """Clone environment dialog."""
        super(CloneDialog, self).__init__(parent=parent)

        # Widgets
        self.label_name = widgets.LabelBase("Name:")
        self.text_name = common.LineEditEnvironment()

        self.label_location = widgets.LabelBase("Location:")
        self.label_prefix = widgets.LabelBase()
        self.button_ok = widgets.ButtonPrimary('Clone')
        self.button_cancel = widgets.ButtonNormal('Cancel')

        # Widget setup
        self.align_labels([self.label_name, self.label_location])
        self.setMinimumWidth(self.BASE_DIALOG_WIDTH)
        self.setWindowTitle("Clone from environment: " + clone_from_name)
        self.text_name.setPlaceholderText("New environment name")
        self.label_prefix.setObjectName('environment-location')

        # Layouts
        grid = QtWidgets.QGridLayout()
        grid.addWidget(self.label_name, 2, 0)
        grid.addWidget(widgets.SpacerHorizontal(), 2, 1)
        grid.addWidget(self.text_name, 2, 2)
        grid.addWidget(widgets.SpacerVertical(), 3, 0)
        grid.addWidget(self.label_location, 4, 0)
        grid.addWidget(self.label_prefix, 4, 2)

        layout_buttons = QtWidgets.QHBoxLayout()
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_cancel)
        layout_buttons.addWidget(widgets.SpacerHorizontal())
        layout_buttons.addWidget(self.button_ok)

        layout = QtWidgets.QVBoxLayout()
        layout.addLayout(grid)
        layout.addWidget(widgets.SpacerVertical())
        layout.addWidget(widgets.SpacerVertical())
        layout.addLayout(layout_buttons)

        self.setLayout(layout)

        # Signals
        self.text_name.textChanged.connect(self.refresh)
        self.button_ok.clicked.connect(self.accept)
        self.button_cancel.clicked.connect(self.reject)

        # Setup
        self.text_name.setFocus()
        self.refresh()

    def refresh(self, text=''):
        """Update status of buttons based on combobox selection."""
        name = self.name
        self.update_location()
        if self.environments:
            self.button_ok.setDisabled(not self.is_valid_env_name(name))



#!/opt/concourse/worker/volumes/live/2bcbe701-7616-43e6-7186-dd9f074f408f/volume/anaconda-navigator_1632168928028/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/2bcbe701-7616-43e6-7186-dd9f074f408f/volume/anaconda-navigator_1632168928028/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])



#!/usr/bin/python
# -*- coding: utf-8 -*-
###############################################################################
#
# Formatting filter for urllib2's HTTPHandler(debuglevel=1) output
# Copyright (c) 2013, Analytics Pros
#
# This project is free software, distributed under the BSD license.
# Analytics Pros offers consulting and integration services if your firm needs
# assistance in strategy, implementation, or auditing existing work.
#
###############################################################################
"""HTTP log used in testng."""

# yapf: disable

from __future__ import division, print_function, with_statement

# Standard library imports
import re
import sys

# Third party imports
from six.moves import cStringIO as StringIO  # Used by tests
import six


# yapf: enable

StringIO


class BufferTranslator(object):
    """Provides a buffer-compatible interface for filtering buffer content."""

    parsers = []

    @staticmethod
    def stripslashes(content):
        """Strip slashes."""
        if six.PY3:
            content = content.encode('UTF-8')
            return content.decode('unicode_escape')
        else:
            return content.decode('string_escape')

    @staticmethod
    def addslashes(content):
        """Add slashes."""
        if six.PY3:
            return content.encode('unicode_escape')
        else:
            return content.encode('string_escape')

    def __init__(self, output):
        """Init."""
        self.output = output
        self.encoding = getattr(output, 'encoding', None)

    def write(self, content):
        """Write."""
        content = self.translate(content)
        self.output.write(content)

    def translate(self, line):
        """Translate."""
        for pattern, method in self.parsers:
            match = pattern.match(line)
            if match:
                return method(match)
        return line

    def flush(self):
        """Flush."""
        pass


class LineBufferTranslator(BufferTranslator):
    """
    Line buffer implementation supports translation of line-format input.

    Works even when input is not already line-buffered. Caches input until
    newlines occur, and then dispatches translated input to output buffer.
    """
    def __init__(self, *args, **kwargs):
        """Init."""
        self._linepending = []
        super(LineBufferTranslator, self).__init__(*args, **kwargs)

    def write(self, _input):
        """Write."""
        lines = _input.splitlines(True)
        last = 0
        for i in range(0, len(lines)):
            last = i
            if lines[i].endswith('\n'):
                prefix = (len(self._linepending) and ''.join(self._linepending) or '')
                self.output.write(self.translate(prefix + lines[i]))
                del self._linepending[0:]
                last = -1

        if lines and last >= 0:
            self._linepending.append(lines[last])

    def __del__(self):
        """Delete method."""
        if len(self._linepending):
            self.output.write(self.translate(''.join(self._linepending)))


class HTTPTranslator(LineBufferTranslator):
    """
    Translates output from |urllib2| HTTPHandler(debuglevel = 1).

    It translates into HTTP-compatible, readible text structures for human
    analysis.
    """

    RE_LINE_PARSER = re.compile(r'^(?:([a-z]+):)\s*(\'?)([^\r\n]*)\2(?:[\r\n]*)$')
    RE_LINE_BREAK = re.compile(r'(\r?\n|(?:\\r)?\\n)')
    RE_HTTP_METHOD = re.compile(r'^(POST|GET|HEAD|DELETE|PUT|TRACE|OPTIONS)')
    RE_PARAMETER_SPACER = re.compile(r'&([a-z0-9]+)=')

    @classmethod
    def spacer(cls, line):
        """Spacer."""
        return cls.RE_PARAMETER_SPACER.sub(r' &\1= ', line)

    def translate(self, line):
        """Translate."""
        parsed = self.RE_LINE_PARSER.match(line)

        if parsed:
            value = parsed.group(3)
            stage = parsed.group(1)

            if stage == 'send':  # query string is rendered here
                return '\n# HTTP Request:\n' + self.stripslashes(value)
            elif stage == 'reply':
                return '\n\n# HTTP Response:\n' + self.stripslashes(value)
            elif stage == 'header':
                return value + '\n'
            else:
                return value

        return line


def consume(outbuffer=None):
    """Capture standard output."""
    sys.stdout = HTTPTranslator(outbuffer or sys.stdout)
    return sys.stdout


if __name__ == '__main__':
    consume(sys.stdout).write(sys.stdin.read())
    print('\n')
